module ParserCodeMessage

  def get_mensagem_ocorrencia code
    case code
      when "02"
       "Entrada Confirmada"
      when "03"
       "Entrada Rejeitada"
      when "06"
       "Liquidação normal (sem motivo)"
      when "09"
       "Baixado Automat. via Arquivo"
      when "10"
        "Baixado conforme instruções da Agência"
      when "11"
        "Em Ser - Arquivo de Títulos pendentes (sem motivo)"
      when "12"
        "Abatimento Concedido (sem motivo)"
      when "13"
        "Abatimento Cancelado (sem motivo)"
      when "14"
        "Vencimento Alterado (sem motivo)"
      when "15"
        "Liquidação em Cartório (sem motivo)"
      when "16"
        "Título Pago em Cheque – Vinculado"
      when "17"
        "Liquidação após baixa ou Título não registrado (sem motivo)"
      when "18"
        "Acerto de Depositária (sem motivo)"
      when "19"
        "Confirmação Receb. Inst. de Protesto"
      when "20"
        "Confirmação Recebimento Instrução Sustação de Protesto (sem motivo)"
      when "21"
        "Acerto do Controle do Participante (sem motivo)"
      when "22"
        "Título Com Pagamento Cancelado"
      when "23"
        "Entrada do Título em Cartório (sem motivo)"
      when "24"
        "Entrada rejeitada por CEP Irregular" 
      when "25"
        "Confirmação Receb.Inst.de Protesto Falimentar"
      when "27"
        "Baixa Rejeitada"
      when "28"
        "Débito de tarifas/custas"
      when "29"
        "Ocorrências do Pagador"
      when "30"
        "Alteração de Outros Dados Rejeitados"
      when "32"
        "Instrução Rejeitada"
      when "33"
        "Confirmação Pedido Alteração Outros Dados (sem motivo)"
      when "34"
        "Retirado de Cartório e Manutenção Carteira (sem motivo)"
      when "35"
        "Desagendamento do débito automático"
      when "40"
        "Estorno de pagamento"
      when "55"
        "Sustado judicial"
      when "68"
        "Acerto dos dados do rateio de Crédito"
      when "69"
        "Cancelamento dos dados do rateio"
    end
  end

  def get_motivo_ocorrencia codigo_ocorrencia, codigo_mensagem
    case codigo_ocorrencia
      when "02"
        parse_entrada_confirmada codigo_mensagem
      when "03"
        parse_entrada_rejeitada codigo_mensagem
      when "06"
        parse_liquidacao codigo_mensagem
      when "09"
        parse_baixa_automatica codigo_mensagem
      when "10"
        parse_baixa_banco codigo_mensagem
      when "15"
        parse_liquidacao_cartorio codigo_mensagem
      when "17"
        parse_liquidacao_baixa codigo_mensagem
      when "24"
        "CEP inválido"
      when "27"
        parse_baixa_rejeitada codigo_mensagem
      when "30"
        parse_dados_rejeitados codigo_mensagem
      when "32"
        parse_instrucao_rejeitada codigo_mensagem
      else
        "#{codigo_ocorrencia} não possui mensagem correspondente"
    end
  end

  def parse_entrada_cofirmada code
    case code
      when "00"
        "Ocorrência aceita"
      when "01"
        "Código do Banco inválido"
      when "04"
        "Código do movimento não permitido para a carteira"
      when "15"
        "Características da cobrança incompatíveis"
      when "17"
        "Data de vencimento anterior a data de emissão"
      when "21"
        "Espécie do Título inválido"
      when "24"
        "Data da emissão inválida"
      when "27"
        "Valor/taxa de juros mora inválido"
      when "38"
        "Prazo para protesto inválido"
      when "39"
        "Pedido para protesto não permitido para título"
      when "43"
        "Prazo para baixa e devolução inválido"
      when "45"
        "Nome do Pagador inválido"
      when "46"
        "Tipo/num. de inscrição do Pagador inválidos"
      when "47"
        "Endereço do Pagador não informado"
      when "48"
        "CEP Inválido"
      when "50"
        "CEP referente a Banco correspondente"
      when "53"
        "No de inscrição do Pagador/avalista inválidos (CPF/CNPJ)"
      when "54"
        "Pagadorr/avalista não informado"
      when "67"
        "Débito automático agendado"
      when "68"
        "Débito não agendado - erro nos dados de remessa"
      when "69"
        "Débito não agendado - Pagador não consta no cadastro de autorizante"
      when "70"
        "Débito não agendado - Beneficiário não autorizado pelo Pagador"
      when "71"
        "Débito não agendado - Beneficiário não participa da modalidade de déb.automático"
      when "72"
        "Débito não agendado - Código de moeda diferente de R$"
      when "73"
        "Débito não agendado - Data de vencimento inválida/vencida"
      when "75"
        "Débito não agendado - Tipo do número de inscrição do pagador debitado inválido"
      when "76"
        "Pagador Eletrônico DDA ()- Esse motivo somente será disponibilizado no arquivo retorno
        para as empresas cadastradas nessa condição."
      when "86"
        "Seu número do documento inválido"
      when "89"
        "Email Pagador não enviado – título com débito automático"
      when "90"
        "Email pagador não enviado – título de cobrança sem registro"
      else
        "#{code} não possui mensagem correspondente"
    end
  end

  def parse_entrada_cofirmada code
    case code
      when "02"
        "Código do registro detalhe inválido"
      when "03"
        "Código da ocorrência inválida"
      when "04"
        "Código de ocorrência não permitida para a carteira"
      when "05"
        "Código de ocorrência não numérico"
      when "07"
        "Agência/conta/Digito - |Inválido"
      when "08"
        "Nosso número inválido"
      when "09"
        "Nosso número duplicado"
      when "10"
        "Carteira inválida"
      when "13"
        "Identificação da emissão do bloqueto inválida"
      when "16"
        "Data de vencimento inválida"
      when "18"
        "Vencimento fora do prazo de operação"
      when "20"
        "Valor do Título inválido"
      when "21"
        "Espécie do Título inválida"
      when "22"
        "Espécie não permitida para a carteira"
      when "24"
        "Data de emissão inválida"
      when "28"
        "Código do desconto inválido"
      when "38"
        "Prazo para protesto inválido"
      when "44"
        "Agência Beneficiário não prevista"
      when "45"
        "Nome do pagador não informado"
      when "46"
        "Tipo número de inscrição do pagador inválidos"
      when "47"
        "Endereço do pagador não informado"
      when "48"
        "CEP Inválido"
      when "50"
        "CEP irregular - Banco Correspondente"
      when "63"
        "Entrada para Título já cadastrado"
      when "65"
        "Limite excedido"
      when "66"
        "Número autorização inexistente"
      when "68"
        "Débito não agendado - erro nos dados de remessa"
      when "69"
        "Débito não agendado - Pagador não consta no cadastro de autorizante"
      when "70"
        "Débito não agendado - Beneficiário não autorizado pelo Pagador"
      when "71"
        "Débito não agendado - Beneficiário não participa do débito Automático"
      when "72"
        "Débito não agendado - Código de moeda diferente de R$"
      when "73"
        "Débito não agendado - Data de vencimento inválida"
      when "74"
        "Débito não agendado - Conforme seu pedido, Título não registrado"
      when "75"
        "Débito não agendado – Tipo de número de inscrição do debitado inválido"
      else
        "#{code} não possui mensagem correspondente"
    end
  end

  def parse_liquidacao code
    case code
      when "00"
        "Título pago com dinheiro"
      when "15"
        "Título pago com cheque"
      when "42"
        "Rateio não efetuado"
      else
        "#{code} não possui mensagem correspondente"
    end
  end

  def parse_baixa_automatica code
    case code
      when "00"
        "Ocorrência Aceita"
      when "10"
        "Baixa Comandada pelo cliente"
      else
        "#{code} não possui mensagem correspondente"
    end
  end

  def parse_baixa_banco code
    case code
      when "00"
        "Baixado Conforme Instruções da Agência"
      when "14"
        "Título Protestado"
      when "15"
        "Título excluído"
      when "16"
        "Título Baixado pelo Banco por decurso Prazo"
      when "17"
        "Titulo Baixado Transferido Carteira"
      when "20"
        "Titulo Baixado e Transferido para Desconto"
      else
        "#{code} não possui mensagem correspondente"
    end
  end

  def parse_liquidacao_cartorio code
    case code
      when "00"
        "Título pago com dinheiro"
      when "15"
        "Título pago com cheque"
      else
        "#{code} não possui mensagem correspondente"
    end
  end

  def parse_liquidacao_baixa code
    case code
      when "00"
        "Título pago com dinheiro"
      when "15"
        "Título pago com cheque"
      else
        "#{code} não possui mensagem correspondente"
    end
  end

  def parse_baixa_rejeitada code
    case code
      when "04"
        "Código de ocorrência não permitido para a carteira"
      when "07"
        "Agência/Conta/dígito inválidos"
      when "08"
        "Nosso número inválido"
      when "10"
        "Carteira inválida"
      when "15"
        "Carteira/Agência/Conta/nosso número inválidos"
      when "40"
        "Título com ordem de protesto emitido"
      when "42"
        "Código para baixa/devolução via Tele Bradesco inválido"
      when "60"
        "Movimento para Título não cadastrado"
      when "77"
        "Transferência para desconto não permitido para a carteira"
      when "85"
        "Título com pagamento vinculado"
      else
        "#{code} não possui mensagem correspondente"
    end
  end

  def parse_baixa_rejeitada code
    case code
      when "04"
        "Código de ocorrência não permitido para a carteira"
      when "07"
        "Agência/Conta/dígito inválidos"
      when "08"
        "Nosso número inválido"
      when "10"
        "Carteira inválida"
      when "15"
        "Carteira/Agência/Conta/nosso número inválidos"
      when "40"
        "Título com ordem de protesto emitido"
      when "42"
        "Código para baixa/devolução via Tele Bradesco inválido"
      when "60"
        "Movimento para Título não cadastrado"
      when "77"
        "Transferência para desconto não permitido para a carteira"
      when "85"
        "Título com pagamento vinculado"
      else
        "#{code} não possui mensagem correspondente"
    end
  end

  def parse_dados_rejeitados code
    case code
      when "01"
        "Código do Banco inválido"
      when "04"
        "Código de ocorrência não permitido para a carteira"
      when "05"
        "Código da ocorrência não numérico"
      when "08"
        "Nosso número inválido"
      when "15"
        "Característica da cobrança incompatível"
      when "16"
        "Data de vencimento inválido"
      when "17"
        "Data de vencimento anterior a data de emissão"
      when "18"
        "Vencimento fora do prazo de operação"
      when "24"
        "Data de emissão Inválida"
      when "26"
        "Código de juros de mora inválido" 
      when "27"
        "Valor/taxa de juros de mora inválido" 
      when "28"
        "Código de desconto inválido"
      when "29"
        "Valor do desconto maior/igual ao valor do Título"
      when "30"
        "Desconto a conceder não confere"
      when "31"
        "Concessão de desconto já existente ( Desconto anterior )"
      when "32"
        "Valor do IOF inválido"
      when "33"
        "Valor do abatimento inválido"
      when "34"
        "Valor do abatimento maior/igual ao valor do Título"
      when "38"
        "Prazo para protesto inválido"
      when "39"
        "Pedido de protesto não permitido para o Título"
      when "40"
        "Título com ordem de protesto emitido"
      when "42"
        "Código para baixa/devolução inválido"
      when "46"
        "Tipo/número de inscrição do pagador inválidos"
      when "48"
        "Cep Inválido"
      when "53"
        "Tipo/Número de inscrição do pagador/avalista inválidos"
      when "54"
        "Pagadorr/avalista não informado"
      when "57"
        "Código da multa inválido"
      when "58"
        "Data da multa inválida"
      when "60"
        "Movimento para Título não cadastrado"
      when "79"
        "Data de Juros de mora Inválida"
      when "80"
        "Data do desconto inválida"
      when "85"
        "Título com Pagamento Vinculado."
      when "88"
        "E-mail Pagador não lido no prazo 5 dias"
      when "91"
        "E-mail pagador não recebido"
      else
        "#{code} não possui mensagem correspondente"
    end
  end

  def parse_instrucao_rejeitada code
    case code
      when "01"
        "Código do Banco inválido"
      when "02"
        "Código do registro detalhe inválido"
      when "04"
        "Código de ocorrência não permitido para a carteira"
      when "05"
        "Código de ocorrência não numérico"
      when "07"
        "Agência/Conta/dígito inválidos"
      when "08"
        "Nosso número inválido"
      when "10"
        "Carteira inválida"
      when "15"
        "Características da cobrança incompatíveis"
      when "16"
        "Data de vencimento inválida"
      when "17"
        "Data de vencimento anterior a data de emissão"
      when "18"
        "Vencimento fora do prazo de operação"
      when "20"
        "Valor do título inválido"
      when "21"
        "Espécie do Título inválida"
      when "22"
        "Espécie não permitida para a carteira"
      when "24"
        "Data de emissão inválida"
      when "28"
        "Código de desconto via Telebradesco inválido"
      when "29"
        "Valor do desconto maior/igual ao valor do Título"
      when "30"
        "Desconto a conceder não confere"
      when "31"
        "Concessão de desconto - Já existe desconto anterior"
      when "33"
        "Valor do abatimento inválido"
      when "34"
        "Valor do abatimento maior/igual ao valor do Título"
      when "36"
        "Concessão abatimento - Já existe abatimento anterior"
      when "38"
        "Prazo para protesto inválido"
      when "39"
        "Pedido de protesto não permitido para o Título"
      when "40"
        "Título com ordem de protesto emitido"
      when "41"
        "Pedido cancelamento/sustação para Título sem instrução de protesto"
      when "42"
        "Código para baixa/devolução inválido"
      when "45"
        "Nome do Pagador não informado"
      when "46"
        "Tipo/número de inscrição do Pagador inválidos"
      when "47"
        "Endereço do Pagador não informado"
      when "48"
        "CEP Inválido"
      when "50"
        "CEP referente a um Banco correspondente"
      when "53"
        "Tipo de inscrição do pagador avalista inválidos"
      when "60"
        "Movimento para Título não cadastrado"
      when "85"
        "Título com pagamento vinculado"
      when "86"
        "Seu número inválido"
      when "94"
        "Título Penhorado – Instrução Não Liberada pela Agência"
      else
        "#{code} não possui mensagem correspondente"
    end
  end
end